'use strict';

var path = require('path');
var fs = require('fs-extra');
var JSZip = require('jszip');
var pluginutils = require('@rollup/pluginutils');
var tsickle = require('tsickle');
var ts = require('typescript');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var JSZip__default = /*#__PURE__*/_interopDefaultLegacy(JSZip);

const tsToJs = path => path.replace(/\.tsx?$/i, ".js");
const humanlizePath = filepath => path__default['default'].relative(process.cwd(), filepath);

var index = ((options = {}) => {
  var _options$include, _options$tsconfig;

  const isIncluded = pluginutils.createFilter((_options$include = options.include) !== null && _options$include !== void 0 ? _options$include : /\.tsx?$/i, options.exclude);
  const tsconfig = (_options$tsconfig = options.tsconfig) !== null && _options$tsconfig !== void 0 ? _options$tsconfig : "tsconfig.json";
  const zip = new JSZip__default['default']();
  const emitted = new Map();
  let compilerOptions;
  let compilerHost;
  let tsickleHost;
  const plugin = {
    name: "tsickle",

    buildStart() {
      const loaded = ts.readConfigFile(tsconfig, file => fs__default['default'].readFileSync(file, "utf8"));
      const parsed = ts.parseJsonConfigFileContent(loaded.config, ts.sys, ".", {}, tsconfig);
      compilerOptions = parsed.options;
      if (compilerOptions.paths) for (const [from, to] of Object.entries(compilerOptions.paths)) compilerOptions.paths[from] = to.map(t => path__default['default'].resolve(t).replace(/\\/g, "/"));
      compilerHost = ts.createCompilerHost(compilerOptions);
      tsickleHost = {
        shouldSkipTsickleProcessing: () => false,
        shouldIgnoreWarningsForPath: () => false,
        pathToModuleName: name => name,
        fileNameToModuleId: name => name,
        options: compilerOptions,
        moduleResolutionHost: compilerHost,
        googmodule: false
      };
    },

    transform(_, id) {
      if (!isIncluded(id)) return;
      options.debug && console.log(`TSICKLE - INCLUDED (${humanlizePath(id)})`);
      const _id = id.split("?")[0];

      if (_id !== id && /\.vue$/i.test(_id)) {
        options.debug && console.log(`TSICKLE - VUE SFC (${humanlizePath(id)})`);
        throw new Error("TS inside Vue SFC is not supported for now");
      }

      const sourceFileName = id.replace(/\\/g, "/");
      const program = ts.createProgram([sourceFileName], compilerOptions, compilerHost);
      tsickle.emit(program, tsickleHost, (file, code) => emitted.set(file, code));
      const sourceFileAsJs = tsToJs(sourceFileName);

      for (const [file, source] of emitted) {
        if (!sourceFileAsJs.includes(file)) continue;
        options.debug && zip.file(`individual/${path__default['default'].basename(file)}`, source);
        const sourceRel = humanlizePath(sourceFileName);
        const resRel = path__default['default'].basename(sourceFileAsJs);
        options.debug && console.log(`TSICKLE - PROCESSED (${sourceRel} to ${resRel})`);
        return {
          code: source
        };
      }

      options.debug && console.log(`TSICKLE - MISSED (${humanlizePath(id)} in ${JSON.stringify(emitted)})`);
      return null;
    },

    renderChunk(code, chunk) {
      options.debug && zip.file(chunk.fileName, code);
      return null;
    },

    async generateBundle() {
      options.debug && this.emitFile({
        type: "asset",
        name: "tsickle.zip",
        source: await zip.generateAsync({
          type: "uint8array"
        })
      });
    }

  };
  return plugin;
});

module.exports = index;
